const request = require('request');
const AsksuiteProcessingNaturalUtils = require('../AsksuiteProcessingNaturalUtils');
const { ErrorHandlerService } = require('./ErrorHandlerService');
const Util = require('../util');
const QuoteDataAccessor = require('./QuoteDataAccessor');
const AWSLambdaCaller = require('./AWSLambdaCaller');
const RequestPreprocessor = require('../util/RequestPreprocessor');

class NLPClassifierAccessor {
  constructor(NLP_CLASSIFIER_LAYER_URL) {
    this.NLP_CLASSIFIER_LAYER_URL = NLP_CLASSIFIER_LAYER_URL;
    this.supportedLanguages = ['pt', 'es', 'en'];
    this.utils = new AsksuiteProcessingNaturalUtils();
  }

  resolveText(request) {
    return new Promise(resolve => {
      const executor = async () => {
        let response = null;
        let dialog = null;
        try {
          response = await this.requestData(this.NLP_CLASSIFIER_LAYER_URL, {
            phrase: request.text,
            language: request.language.substring(0, 2),
            intentType: 'cotacao',
          });

          const intent = response.body.intent;

          if (!intent) {
            throw { message: response.statusMessage, code: response.statusCode };
          }

          dialog = this.utils.findDialogByIntent(request, intent);

          if (Util.isQuote(intent)) {
            try {
              dialog.extractedData = await this.extractQuoteData(request);
            } catch (err) {
              console.log('[NLPClassifierAccessor error] Trying extract Quote Data', err);
            }
          }

          dialog.processingStep = {
            ...dialog.processingStep,
            text: request.text,
            language: request.language,
            resolver: 'NLP_CLASSIFIER',
            extraInfo: { ...response.body.trackingInfo },
          };
        } catch (e) {
          const errorData = ErrorHandlerService.accessorErrorHandler(e);
          dialog = {
            processingStep: {
              fallbackReason: 'intent_not_found',
              intentFound: null,
              text: request.text,
              language: request.language,
              resolver: 'NLP_CLASSIFIER',
              errorData,
            },
          };
          console.log('[NLPClassifierAccessor error]', e);
        }
        resolve(dialog);
      };

      executor();
    });
  }

  requestData(url, json) {
    return new Promise((resolve, reject) => {
      request.post(url, { json, timeout: 20 * 1000 }, (err, res) => {
        if (err) {
          reject(err);
        }
        resolve(res);
      });
    });
  }

  isSupportedLanguage(language) {
    return this.supportedLanguages.some(
      supportedLanguage => supportedLanguage === language.substring(0, 2),
    );
  }

  async extractQuoteData(request) {
    request = await RequestPreprocessor.process(request);

    const awsLambdaCaller = new AWSLambdaCaller(request.config.CORE_LAMBDA_AWS);
    const quoteDataAccessor = new QuoteDataAccessor(awsLambdaCaller);

    return quoteDataAccessor.resolveText(request);
  }
}

module.exports = NLPClassifierAccessor;
